import { AccessFlags } from '@azure-tools/typespec-client-generator-core';
import { CreateSdkContextOptions } from '@azure-tools/typespec-client-generator-core';
import { DateTimeKnownEncoding } from '@typespec/compiler';
import { DecoratorInfo } from '@azure-tools/typespec-client-generator-core';
import { Diagnostic } from '@typespec/compiler';
import { DiagnosticDefinition } from '@typespec/compiler';
import { DiagnosticMessages } from '@typespec/compiler';
import { DiagnosticReport } from '@typespec/compiler';
import { DurationKnownEncoding } from '@typespec/compiler';
import { EmitContext } from '@typespec/compiler';
import { JSONSchemaType } from '@typespec/compiler';
import { Program } from '@typespec/compiler';
import { SdkBuiltInKinds } from '@azure-tools/typespec-client-generator-core';
import { SdkContext } from '@azure-tools/typespec-client-generator-core';
import { SdkEmitterOptions } from '@azure-tools/typespec-client-generator-core';
import { SdkType } from '@azure-tools/typespec-client-generator-core';
import { SerializationOptions } from '@azure-tools/typespec-client-generator-core';
import { Tracer } from '@typespec/compiler';
import { UsageFlags } from '@azure-tools/typespec-client-generator-core';

/* Excluded from this release type: $onEmit */

declare type ApiKeyLocation = "header";

declare enum BodyMediaType {
    None = "None",
    Binary = "Binary",
    Form = "Form",
    Json = "Json",
    Multipart = "Multipart",
    Text = "Text",
    Xml = "Xml"
}

declare interface CodeModel {
    Name: string;
    ApiVersions: string[];
    Enums: InputEnumType[];
    Models: InputModelType[];
    Clients: InputClient[];
    Auth?: InputAuth;
}

/* Excluded from this release type: configurationFileName */

/* Excluded from this release type: createDiagnostic */

/* Excluded from this release type: createModel */

/* Excluded from this release type: CSharpEmitterContext */

/* Excluded from this release type: CSharpEmitterOptions */

/* Excluded from this release type: CSharpEmitterOptionsSchema */

/* Excluded from this release type: defaultOptions */

declare type DiagnosticMessagesMap = {
    [K in keyof typeof diags]: (typeof diags)[K]["messages"];
};

declare const diags: {
    [code: string]: DiagnosticDefinition<DiagnosticMessages>;
};

/* Excluded from this release type: getTracer */

declare interface HttpResponseHeader {
    Name: string;
    NameInResponse: string;
    Summary: string;
    Doc: string;
    Type: InputType;
}

declare interface InputApiKeyAuth {
    Name: string;
    In: ApiKeyLocation;
    Prefix?: string;
}

declare interface InputArrayExampleValue extends InputExampleTypeValueBase {
    kind: "array";
    type: InputArrayType;
    value: InputExampleValue[];
}

declare interface InputArrayType extends InputTypeBase {
    kind: "array";
    name: string;
    valueType: InputType;
    crossLanguageDefinitionId: string;
}

declare interface InputAuth {
    ApiKey?: InputApiKeyAuth;
    OAuth2?: InputOAuth2Auth;
}

declare interface InputBooleanExampleValue extends InputExampleTypeValueBase {
    kind: "boolean";
    type: InputPrimitiveType | InputLiteralType;
    value: boolean;
}

declare interface InputClient {
    Name: string;
    ClientNamespace: string;
    Summary?: string;
    Doc?: string;
    Operations: InputOperation[];
    Protocol?: Protocols;
    Parent?: string;
    Parameters?: InputParameter[];
    Decorators?: DecoratorInfo[];
}

declare interface InputConstant {
    Value?: any;
    Type: InputType;
}

declare type InputDateTimeType = InputUtcDateTimeType | InputOffsetDateTimeType;

declare interface InputDateTimeTypeBase extends InputTypeBase {
    name: string;
    encode: DateTimeKnownEncoding;
    wireType: InputPrimitiveType;
    crossLanguageDefinitionId: string;
    baseType?: InputDateTimeType;
}

declare interface InputDictionaryExampleValue extends InputExampleTypeValueBase {
    kind: "dict";
    type: InputDictionaryType;
    value: Record<string, InputExampleValue>;
}

declare interface InputDictionaryType extends InputTypeBase {
    kind: "dict";
    keyType: InputType;
    valueType: InputType;
}

declare interface InputDurationType extends InputTypeBase {
    kind: "duration";
    name: string;
    encode: DurationKnownEncoding;
    wireType: InputPrimitiveType;
    crossLanguageDefinitionId: string;
    baseType?: InputDurationType;
}

declare interface InputEnumType extends InputTypeBase {
    kind: "enum";
    name: string;
    crossLanguageDefinitionId: string;
    valueType: InputPrimitiveType;
    values: InputEnumTypeValue[];
    isFixed: boolean;
    isFlags: boolean;
    usage: UsageFlags;
    access?: AccessFlags;
    clientNamespace: string;
}

declare interface InputEnumTypeValue extends InputTypeBase {
    kind: "enumvalue";
    name: string;
    value: string | number;
    enumType: InputEnumType;
    valueType: InputPrimitiveType;
}

declare interface InputExampleBase {
    kind: string;
    name: string;
    description: string;
    filePath: string;
}

declare interface InputExampleTypeValueBase {
    kind: string;
    type: InputType;
    value: unknown;
}

declare type InputExampleValue = InputStringExampleValue | InputNumberExampleValue | InputBooleanExampleValue | InputNullExampleValue | InputUnknownExampleValue | InputArrayExampleValue | InputDictionaryExampleValue | InputUnionExampleValue | InputModelExampleValue;

declare interface InputHttpOperationExample extends InputExampleBase {
    kind: "http";
    parameters: InputParameterExampleValue[];
    responses: OperationResponseExample[];
}

declare interface InputLiteralType extends InputTypeBase {
    kind: "constant";
    valueType: InputPrimitiveType | InputEnumType;
    value: string | number | boolean | null;
}

declare interface InputModelExampleValue extends InputExampleTypeValueBase {
    kind: "model";
    type: InputModelType;
    value: Record<string, InputExampleValue>;
    additionalPropertiesValue?: Record<string, InputExampleValue>;
}

declare interface InputModelProperty extends InputTypeBase {
    kind: "property";
    name: string;
    serializedName: string;
    type: InputType;
    optional: boolean;
    readOnly: boolean;
    discriminator: boolean;
    crossLanguageDefinitionId: string;
    flatten: boolean;
    serializationOptions: SerializationOptions;
}

declare interface InputModelType extends InputTypeBase {
    kind: "model";
    properties: InputModelProperty[];
    name: string;
    crossLanguageDefinitionId: string;
    access?: AccessFlags;
    usage: UsageFlags;
    clientNamespace: string;
    additionalProperties?: InputType;
    discriminatorValue?: string;
    discriminatedSubtypes?: Record<string, InputModelType>;
    discriminatorProperty?: InputModelProperty;
    baseModel?: InputModelType;
    serializationOptions: SerializationOptions;
}

declare interface InputNullableType extends InputTypeBase {
    kind: "nullable";
    type: InputType;
    clientNamespace: string;
}

declare interface InputNullExampleValue extends InputExampleTypeValueBase {
    kind: "null";
    type: InputNullableType;
    value: null;
}

declare interface InputNumberExampleValue extends InputExampleTypeValueBase {
    kind: "number";
    type: InputType;
    value: number;
}

declare interface InputOAuth2Auth {
    Scopes?: string[];
}

declare interface InputOffsetDateTimeType extends InputDateTimeTypeBase {
    kind: "offsetDateTime";
}

declare interface InputOperation {
    Name: string;
    ResourceName?: string;
    Summary?: string;
    Deprecated?: string;
    Doc?: string;
    Accessibility?: string;
    Parameters: InputParameter[];
    Responses: OperationResponse[];
    HttpMethod: RequestMethod;
    RequestBodyMediaType: BodyMediaType;
    Uri: string;
    Path: string;
    ExternalDocsUrl?: string;
    RequestMediaTypes?: string[];
    BufferResponse: boolean;
    LongRunning?: OperationLongRunning;
    Paging?: OperationPaging;
    GenerateProtocolMethod: boolean;
    GenerateConvenienceMethod: boolean;
    Examples?: InputHttpOperationExample[];
    CrossLanguageDefinitionId: string;
    Decorators?: DecoratorInfo[];
}

declare enum InputOperationParameterKind {
    Method = "Method",
    Client = "Client",
    Constant = "Constant",
    Flattened = "Flattened",
    Spread = "Spread",
    Grouped = "Grouped"
}

declare interface InputParameter {
    Name: string;
    NameInRequest: string;
    Summary?: string;
    Doc?: string;
    Type: InputType;
    Location: RequestLocation;
    DefaultValue?: InputConstant;
    VirtualParameter?: VirtualParameter;
    GroupedBy?: InputParameter;
    Kind: InputOperationParameterKind;
    IsRequired: boolean;
    IsApiVersion: boolean;
    IsResourceParameter: boolean;
    IsContentType: boolean;
    IsEndpoint: boolean;
    SkipUrlEncoding: boolean;
    Explode: boolean;
    ArraySerializationDelimiter?: string;
    HeaderCollectionPrefix?: string;
    Decorators?: DecoratorInfo[];
}

declare interface InputParameterExampleValue {
    parameter: InputParameter;
    value: InputExampleValue;
}

declare interface InputPrimitiveType extends InputTypeBase {
    kind: SdkBuiltInKinds;
    name: string;
    encode?: string;
    crossLanguageDefinitionId: string;
    baseType?: InputPrimitiveType;
}

declare interface InputStringExampleValue extends InputExampleTypeValueBase {
    kind: "string";
    type: InputType;
    value: string;
}

declare type InputType = InputPrimitiveType | InputDateTimeType | InputDurationType | InputLiteralType | InputUnionType | InputModelType | InputEnumType | InputArrayType | InputDictionaryType | InputNullableType;

declare interface InputTypeBase {
    kind: string;
    summary?: string;
    doc?: string;
    deprecation?: string;
    decorators?: DecoratorInfo[];
}

declare interface InputUnionExampleValue extends InputExampleTypeValueBase {
    kind: "union";
    type: InputUnionType;
    value: unknown;
}

declare interface InputUnionType extends InputTypeBase {
    kind: "union";
    name: string;
    variantTypes: InputType[];
    clientNamespace: string;
}

declare interface InputUnknownExampleValue extends InputExampleTypeValueBase {
    kind: "unknown";
    type: InputPrimitiveType;
    value: unknown;
}

declare interface InputUtcDateTimeType extends InputDateTimeTypeBase {
    kind: "utcDateTime";
}

/* Excluded from this release type: Logger */

/* Excluded from this release type: LoggerLevel */

declare enum OperationFinalStateVia {
    AzureAsyncOperation = 0,
    Location = 1,
    OriginalUri = 2,
    OperationLocation = 3,
    CustomLink = 4,
    CustomOperationReference = 5,
    NoResult = 6
}

declare interface OperationLongRunning {
    FinalStateVia: OperationFinalStateVia;
    FinalResponse: OperationResponse;
    ResultPath?: string;
}

declare interface OperationPaging {
    NextLinkName?: string;
    ItemName?: string;
    NextLinkOperation?: InputOperation;
    NextLinkOperationRef?: (p: any) => void;
}

declare interface OperationResponse {
    StatusCodes: number[];
    BodyType?: InputType;
    BodyMediaType: BodyMediaType;
    Headers: HttpResponseHeader[];
    ContentTypes?: string[];
    IsErrorResponse: boolean;
}

declare interface OperationResponseExample {
    response: OperationResponse;
    statusCode: number;
    bodyValue?: InputExampleValue;
}

declare interface Protocols {
}

/* Excluded from this release type: reportDiagnostic */

declare enum RequestLocation {
    None = "",
    Uri = "Uri",
    Path = "Path",
    Query = "Query",
    Header = "Header",
    Body = "Body"
}

declare enum RequestMethod {
    GET = "GET",
    POST = "POST",
    PUT = "PUT",
    PATCH = "PATCH",
    DELETE = "DELETE",
    HEAD = "HEAD",
    OPTIONS = "OPTIONS",
    TRACE = "TRACE",
    NONE = ""
}

/* Excluded from this release type: resolveOptions */

declare interface SdkTypeMap {
    types: Map<SdkType, InputType>;
    models: Map<string, InputModelType>;
    enums: Map<string, InputEnumType>;
}

/* Excluded from this release type: setSDKContextOptions */

/* Excluded from this release type: tspOutputFileName */

declare interface VirtualParameter {
}

/* Excluded from this release type: writeCodeModel */

export { }
